package config

import (
	"time"
)

type Config struct {
	Postgres PostgresConfig `env:",prefix=POSTGRES_"`
	HTTP     HTTP           `env:",prefix=HTTP_"`
	Log      LogConfig      `env:",prefix=LOG_"`
	Security Security       `env:",prefix=SECURITY_"`
	Fetcher  Fetcher        `env:",prefix=FETCHER_"`
}

type Security struct {
	JWKSURL     string `env:"JWKS_URL" validate:"uri,required"`
	UserInfoURL string `env:"USER_INFO_URL" validate:"uri,optional"`
}

type HTTP struct {
	Listen    string `env:"LISTEN,default=0.0.0.0:8080"`
	APIPrefix string `env:"API_PREFIX"`
	AccessLog bool   `env:"ENABLE_ACCESS_LOG,default=false"`
	EnableGUI bool   `env:"ENABLE_GUI,default=true"` // TODO
}

type PostgresConfig struct {
	DSN string `env:"DSN" validate:"uri"`
}

type Fetcher struct {
	Interval time.Duration `env:"INTERVAL,default=1m"`
}

type LogConfig struct {
	// possible values: DEBUG INFO WARN ERROR
	Level string `env:"LEVEL,default=INFO" validate:"oneof=DEBUG INFO WARN ERROR"`
	// possible values:
	// * "none"
	// * "stdout"
	// * "loki+http://localhost:3100"
	Output string `env:"OUTPUT,default=stdout"`
}
