package dto

import (
	"fmt"
	"time"

	"github.com/google/uuid"
	"gopkg.in/guregu/null.v4"
)

type Provider string

func (e *Provider) Scan(src interface{}) error {
	switch s := src.(type) {
	case []byte:
		*e = Provider(s)
	case string:
		*e = Provider(s)
	default:
		return fmt.Errorf("unsupported scan type for Provider: %T", src)
	}
	return nil
}

const (
	ProviderRSS Provider = "rss"
)

type FeedState struct {
	LastFetchedAt time.Time     `json:"-"`
	FetchInterval time.Duration `json:"-"`
	LastError     null.String   `json:"-"`
	ErrorsCount   int           `json:"-"`
}

type Feed struct {
	ID       uuid.UUID `json:"id"`
	Title    string    `json:"title"`
	Provider Provider  `json:"provider"`
	Source   string    `json:"source"`

	UnreadCount int `json:"unread"`

	State FeedState `json:"-"`
}

type CreateFeed struct {
	Title    null.String `json:"title" validate:"optional"`
	Provider Provider    `json:"provider" validate:"required,oneof=rss"`
	Source   string      `json:"source" validate:"required"`
}

type UpdateFeed struct {
	ID                      uuid.UUID
	Title                   null.String
	FetchIntervalMultiplier float64
	LastFetchedAt           time.Time
}

type UpdateFeedError struct {
	ID            uuid.UUID
	Error         string
	LastFetchedAt time.Time
}

type Item struct {
	ID        uuid.UUID `json:"id"`
	UID       string    `json:"uid"`
	CreatedAt time.Time `json:"createdAt"`
	FetchedAt time.Time `json:"fetchedAt"`
	Title     string    `json:"title"`
	Content   string    `json:"content"`
	Link      string    `json:"link"`
	Author    string    `json:"author"`
	//Tags      []string  `json:"tags"` // TODO: user-defined tags
	//Attachments []any `json:"attachments"` // TODO
}
