package fetcher

import (
	"log/slog"

	"borodyadka.dev/borodyadka/minireader/internal/pkg/validator"
)

type Option func(fetcher *FetchScheduler)

type Repositories struct {
	Feed feedRepository `validate:"required"`
	Item itemRepository `validate:"required"`
}

func WithRepositories(repo Repositories) Option {
	return func(fetcher *FetchScheduler) {
		if err := validator.Struct(repo); err != nil {
			panic("one or more repositories passed into FetchScheduler instance is invalid or not defined")
		}

		fetcher.feeds = repo.Feed
		fetcher.items = repo.Item
	}
}

func WithConfig(config Config) Option {
	return func(fetcher *FetchScheduler) {
		fetcher.checkInterval = config.CheckInterval
		fetcher.fetchers = config.Fetchers
	}
}

func WithLogger(logger *slog.Logger) Option {
	return func(fetcher *FetchScheduler) {
		fetcher.logger = logger
	}
}
