package rss

import (
	"context"
	"net/http"
	"time"

	"github.com/mmcdole/gofeed"

	"borodyadka.dev/borodyadka/minireader/internal/minireader/dto"
	"borodyadka.dev/borodyadka/minireader/internal/pkg/utils"
)

type Config struct {
	HTTPTimeout time.Duration
}

type Fetcher struct {
	config Config
}

func New(config Config) *Fetcher {
	return &Fetcher{config: config}
}

func (f *Fetcher) Fetch(ctx context.Context, source string) (dto.Feed, []dto.Item, error) {
	cl := http.Client{Timeout: f.config.HTTPTimeout}

	req, err := http.NewRequest("GET", source, nil)
	if err != nil {
		return dto.Feed{}, nil, err
	}

	resp, err := cl.Do(req)
	if err != nil {
		return dto.Feed{}, nil, err
	}
	defer resp.Body.Close()

	feed, err := gofeed.NewParser().Parse(resp.Body)
	if err != nil {
		return dto.Feed{}, nil, err
	}

	now := time.Now()
	items := make([]dto.Item, 0, len(feed.Items))
	for _, item := range feed.Items {
		items = append(items, dto.Item{
			UID:       item.GUID,
			CreatedAt: utils.DerefOrDefault(item.PublishedParsed, now),
			FetchedAt: now,
			Title:     item.Title,
			Content:   item.Content,
			Link:      item.Link,
			// TODO: rest of fields
		})
	}

	return dto.Feed{
		Title:    feed.Title,
		Provider: dto.ProviderRSS,
		Source:   source,
	}, items, nil
}
