package reader

import (
	"log/slog"

	"borodyadka.dev/borodyadka/minireader/internal/pkg/validator"
)

type Option func(reader *Reader)

type Repositories struct {
	Feed         feedRepository         `validate:"required"`
	Subscription subscriptionRepository `validate:"required"`
	Item         itemRepository         `validate:"required"`
	User         userRepository         `validate:"required"`
}

func WithRepositories(repo Repositories) Option {
	return func(reader *Reader) {
		if err := validator.Struct(repo); err != nil {
			panic("one or more repositories passed into Reader instance is invalid or not defined")
		}

		reader.feeds = repo.Feed
		reader.subscriptions = repo.Subscription
		reader.items = repo.Item
		reader.users = repo.User
	}
}

func WithFetcher(fetcher fetcher) Option {
	return func(reader *Reader) {
		reader.fetcher = fetcher
	}
}

func WithLogger(logger *slog.Logger) Option {
	return func(reader *Reader) {
		reader.logger = logger
	}
}
