package repositories

import (
	"context"
	"sync/atomic"

	"github.com/google/uuid"
	"github.com/jackc/pgx/v5/pgtype"

	"borodyadka.dev/borodyadka/minireader/internal/minireader/dto"
	"borodyadka.dev/borodyadka/minireader/internal/minireader/repositories/sql"
	"borodyadka.dev/borodyadka/minireader/internal/pkg/utils"
)

type ItemRepository struct {
}

func NewItemRepository() *ItemRepository {
	return &ItemRepository{}
}

func (r *ItemRepository) UpsertItems(ctx context.Context, feed uuid.UUID, items []dto.Item) (int, error) {
	input := utils.MapSlice(items, func(t dto.Item) sql.UpsertFeedItemBatchParams {
		return sql.UpsertFeedItemBatchParams{
			FeedID:    feed,
			CreatedAt: pgtype.Timestamptz{Time: t.CreatedAt, Valid: true},
			FetchedAt: pgtype.Timestamptz{Time: t.FetchedAt, Valid: true},
			UID:       t.UID,
			Title:     t.Title,
			Content:   t.Content,
			Link:      t.Link,
			Author:    t.Author,
		}
	})

	res := sql.New(ConnFromContext(ctx)).UpsertFeedItemBatch(ctx, input)

	var inserted int64
	res.QueryRow(func(_ int, created bool, err error) {
		if created {
			atomic.AddInt64(&inserted, 1)
		}
		if err != nil {
			// TODO: handle error
		}
	})

	if err := res.Close(); err != nil {
		return 0, err
	}

	return int(inserted), nil
}
