-- name: ListOutdatedFeeds :many
SELECT "id", "provider", "source"
FROM "feeds"
WHERE NOW() > "next_fetch_at"
ORDER BY "next_fetch_at"
LIMIT 1000;

-- name: ListAllUserFeeds :many
WITH "counters" AS (SELECT fi."feed_id", si."user_id", COUNT(*) AS "unread"
                    FROM "feed_items" AS fi
                             LEFT JOIN "subscription_items" AS si
                                       ON (fi."feed_id" = "si".feed_id AND fi."id" = si."item_id")
                    WHERE si."is_readed" = FALSE
                       OR si."is_readed" IS NULL
                    GROUP BY fi."feed_id", si."user_id")
SELECT f."id",
       COALESCE(s."title", f."title") AS "title",
       f."provider",
       f."source",
       COALESCE(c."unread", 0)        AS "unread"
FROM "subscriptions" AS s
         JOIN "feeds" AS f ON (s."feed_id" = f."id")
         LEFT JOIN "counters" AS c ON (c."feed_id" = f."id" AND (s."user_id" = c."user_id" OR c."user_id" IS NULL))
WHERE s."user_id" = @user_id;

-- name: FindOrCreateFeed :one
INSERT INTO "feeds" ("title", "provider", "source")
VALUES (COALESCE(@title, 'unnamed feed'), @provider, @source)
ON CONFLICT ("source") DO NOTHING
RETURNING (xmax = 0)::BOOLEAN AS "created", "id", "title", "last_fetched_at", "fetch_interval", "last_error", "errors_count";

-- name: UpdateFeed :exec
UPDATE "feeds"
SET "title"           = COALESCE(NULLIF(@title, ''), "title"),
    "fetch_interval"  = LEAST(GREATEST("fetch_interval" * @interval_multiplier::FLOAT, '1 minute'::interval), '12 hours'::interval),
    "last_fetched_at" = @last_fetched_at
WHERE "id" = @id;

-- name: SaveFeedError :exec
UPDATE "feeds"
SET "last_error"      = @last_error::TEXT,
    "errors_count"    = "errors_count" + 1,
    "last_fetched_at" = @last_fetched_at
WHERE "id" = @id;

-- name: DeleteFeed :execresult
DELETE
FROM "feeds"
WHERE "id" = @id;

-- name: UpsertFeedItemBatch :batchone
INSERT INTO "feed_items" ("feed_id", "created_at", "fetched_at", "uid", "title", "content", "link", "author", "tags")
VALUES (@feed_id::UUID, @created_at::TIMESTAMPTZ, @fetched_at::TIMESTAMPTZ, @uid::TEXT, @title::TEXT, @content::TEXT,
        @link::TEXT, @author::TEXT, '{}'::TEXT[])
ON CONFLICT ("feed_id", "uid") DO UPDATE SET "title"   = @title::TEXT,
                                             "content" = @content::TEXT,
                                             "link"    = @link::TEXT
RETURNING (xmax = 0)::BOOLEAN AS "created";

-- name: CreateSubscription :exec
INSERT INTO "subscriptions" ("user_id", "feed_id")
VALUES (@user_id::TEXT, @feed_id::UUID)
ON CONFLICT ("user_id", "feed_id") DO NOTHING;
