package http

import (
	"github.com/gofiber/fiber/v2"

	"borodyadka.dev/borodyadka/minireader/internal/minireader/dto"
)

func (s *Server) UserInfo(ctx *fiber.Ctx) error {
	user := GetUserFromContext(ctx.Context())

	if err := s.reader.EnsureUser(ctx.Context(), user); err != nil {
		return err
	}

	return ctx.JSON(Result[*User]{
		Data: user,
	})
}

func (s *Server) ImportOPML(ctx *fiber.Ctx) error {
	return nil
}

func (s *Server) ListAllUserFeeds(ctx *fiber.Ctx) error {
	user := GetUserFromContext(ctx.Context())

	feeds, total, err := s.reader.ListAllUserFeeds(ctx.Context(), user)
	if err != nil {
		return err
	}

	return ctx.JSON(Results[Feed]{
		Data: feeds,
		Meta: Meta{
			Total: total,
		},
	})
}

func (s *Server) CreateNewFeed(ctx *fiber.Ctx) error {
	user := GetUserFromContext(ctx.Context())

	var input dto.CreateFeed
	if err := ctx.BodyParser(&input); err != nil {
		return fiber.ErrBadRequest
	}

	feed, err := s.reader.CreateSubscription(ctx.Context(), user, &input)
	if err != nil {
		// TODO: log
		return err
	}

	return ctx.JSON(Result[Feed]{
		Data: feed,
	})
}

func (s *Server) GetFeedInfo(ctx *fiber.Ctx) error {
	return nil
}

func (s *Server) UpdateFeedInfo(ctx *fiber.Ctx) error {
	return nil
}

func (s *Server) DeleteFeed(ctx *fiber.Ctx) error {
	return nil
}

func (s *Server) ListFeedItems(ctx *fiber.Ctx) error {
	return nil
}

func (s *Server) GetItemInfo(ctx *fiber.Ctx) error {
	return nil
}

func (s *Server) MarkItemReaded(ctx *fiber.Ctx) error {
	return nil
}

func (s *Server) MarkItemUnreaded(ctx *fiber.Ctx) error {
	return nil
}
