package http

import (
	"log/slog"

	"borodyadka.dev/borodyadka/minireader/internal/minireader/config"
	"borodyadka.dev/borodyadka/minireader/internal/pkg/validator"
)

type Option func(*Server)

type Services struct {
	Reader reader `validate:"required"`
}

func WithServices(services Services) Option {
	return func(server *Server) {
		if err := validator.Struct(services); err != nil {
			panic("one or more services passed into HTTP Server instance is invalid or not defined")
		}

		server.reader = services.Reader
	}
}

func WithConfig(conf config.HTTP) Option {
	return func(server *Server) {
		server.config = conf
	}
}

func WithSecurity(sec config.Security) Option {
	// TODO: security MUST be a security provider, not a JWKS URL!
	return func(server *Server) {
		server.sec = sec
	}
}

func WithLogger(logger *slog.Logger) Option {
	return func(server *Server) {
		server.logger = logger
	}
}
